package industries.sunshine.planningpoker

import scala.scalajs.js
import com.raquo.laminar.api.L.{*, given}
import industries.sunshine.planningpoker.common.Models.*

object OtherPlayers {
  def render(roomStateSignal: Signal[RoomStateView]): Element = {
    val otherPlayers = roomStateSignal.map { state =>
      state.players.filterNot(_.id == state.me).zipWithIndex
    }
    def playerCardsAmountSignal(id: PlayerID): Signal[Int] = {
      roomStateSignal.map(state => {
        val totalCards = state.allowedCards.size
        val playerModification = state.round match {
          case RoundStateView.Viewing(votes) if votes.toMap.contains(id)           => -1
          case RoundStateView.Voting(_, votedPlayers) if votedPlayers.contains(id) => -1
          case _                                                                   => 0
        }

        totalCards + playerModification
      })
    }

    div(
      className := "flex flex-row",
      children <-- otherPlayers.split(_._1.id) { (playerID, initial, playerSignal) =>
        renderPlayer(playerSignal, playerCardsAmountSignal(playerID))
      }
    )
  }

  def renderPlayer(p: Signal[(Player, Int)], cardsAmount: Signal[Int]): Element = {
    val xOffsetStyleSignal = p.map(_._2)
    div(
      className := "bg-green-200 border-b-2 border-black rounded p-2 m-2 absolute drop-shadow-xl m-3",
      styleAttr <-- p.map(tuple => s"left: ${(0 + tuple._2) * 200}px"),
      child.text <-- p.map(_._1.name),
      renderHandCardBacks(cardsAmount)
    )
  }

  /**
   * adds css variables to be used in tailwindcss classes
   */
  def dynamicCardStyle(index: Int, totalCards: Int): String = {
    val offCenterIndex = 2*index - (totalCards + 1)
    val maxOffCenter = (totalCards + 1) / 2
    val angle = Math.round((offCenterIndex / maxOffCenter.toDouble) * 25)
    val xOffset = (offCenterIndex * 0.2)
    s" --custom-rotate: ${angle}deg; --custom-x-translate: ${xOffset}rem; "
  }

  def renderHandCardBacks(amountSignal: Signal[Int]): Element = {
    def renderCard(index: Int, totalCards: Int): Element =
      div(
        className := "w-8 h-12 rounded bg-green-500 text-yellow m-1 border border-green-700 drop-shadow-md",
        className := "origin-bottom rotate-[--custom-rotate] translate-x-[--custom-x-translate]",
        className := "absolute start-5",
        styleAttr := dynamicCardStyle(index, totalCards),

      )

    div(
      className := "relative h-16 w-20",
      children <-- amountSignal.map { amount => (1 to amount).map(renderCard(_, amount)) }
    )
  }

}
