package industries.sunshine.planningpoker

import scala.scalajs.js
import com.raquo.laminar.api.L.{*, given}
import industries.sunshine.planningpoker.common.Models.*
import io.laminext.websocket.circe.WebSocket._
import io.laminext.websocket.circe.webSocketReceiveBuilderSyntax

import scala.concurrent.duration._

object RoomView {

  /** Rendering of the Room state central table with cards, player control UI and other players
    * description
    *
    * @param loggedIn
    *   \- channel for signaling to the parent about dead websocket, i.e logged out state
    */
  def renderRoom(loggedIn: Observer[Boolean]): Element = {

    val wsStream = io.laminext.websocket.WebSocket
      .path("/api/subscribe")
      .json[RoomStateView, Unit]
      .build(
        managed = true,
        autoReconnect = true,
        reconnectDelay = 500.millis,
        reconnectDelayOffline = 20.seconds,
        reconnectRetries = 3
      )

    val roomStateSignal: Signal[RoomStateView] =
      wsStream.received.startWith(RoomStateView.empty)

    val wsFinalDeathSignal = wsStream.closed.collect { case (_, false) => () }

    div(
      className := "w-full h-full border-4 border-amber-900 flex flex-col relative",
      div(
        className := "absolute top-2 right-2",
        child.text <-- roomStateSignal.map(st => s"Room name: '${st.roomName}'"),
      ),
      OtherPlayers.render(roomStateSignal),
      TableView.renderTable(roomStateSignal),
      OwnHandControls.render(roomStateSignal),
      wsStream.connect,
      wsFinalDeathSignal.map(_ => false) --> loggedIn
    )
  }

}
